//
//  OATOSAuthCredentialProvider.m
//  aliyun_oss_ios
//
//  Created by wombat on 2022/5/12.
//

#import <Foundation/Foundation.h>
#import <AliyunOSSiOS/AliyunOSSiOS.h>
#import "OATOSAuthCredentialProvider.h"

@implementation OATOSAuthCredentialProvider

- (instancetype)initWithAuthServerUrl:(NSString *)authServerUrl auhtorization:(NSString *)authorization
{
  return [self initWithAuthServerUrl:authServerUrl authorization:authorization responseDecoder:nil];
}

- (instancetype)initWithAuthServerUrl:(NSString *)authServerUrl authorization:(NSString*)authorization responseDecoder:(nullable OSSResponseDecoderBlock)decoder
{
  self = [super initWithFederationTokenGetter:^OSSFederationToken * {
    NSURL * url = [NSURL URLWithString:self.authServerUrl];
    NSURLRequest * request = [NSURLRequest requestWithURL:url];
    NSMutableURLRequest *mutableRequest = [request mutableCopy];
    [mutableRequest addValue:authorization forHTTPHeaderField:@"Authorization"];
    request = [mutableRequest copy];
    
    OSSTaskCompletionSource * tcs = [OSSTaskCompletionSource taskCompletionSource];
    NSURLSession * session = [NSURLSession sharedSession];
    NSURLSessionTask * sessionTask = [session dataTaskWithRequest:request
                                                completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
      if (error) {
        [tcs setError:error];
        return;
      }
      [tcs setResult:data];
    }];
    [sessionTask resume];
    [tcs.task waitUntilFinished];
    
    if (tcs.task.error) {
      return nil;
    } else {
      NSData* data = tcs.task.result;
      if(decoder){
        data = decoder(data);
      }
      NSDictionary *object = [NSJSONSerialization JSONObjectWithData:data
                                                              options:kNilOptions
                                                                error:nil];
      NSLog(@"请求结果: %@", [object description]);
      NSString *statusCode = [object objectForKey:@"statusCode"];
      
      if ([[statusCode uppercaseString] isEqualToString:@"OK"]) {
        NSDictionary *data = [object objectForKey:@"data"];
        OSSFederationToken * token = [OSSFederationToken new];
        // All the entries below are mandatory.
        token.tAccessKey = [data objectForKey:@"accessKey"];
        token.tSecretKey = [data objectForKey:@"secretKey"];
        token.tToken = [data objectForKey:@"sessionToken"];
        token.expirationTimeInMilliSecond = [[data objectForKey:@"expiration"] longLongValue];
        OSSLogDebug(@"token: %@ %@ %@ %@", token.tAccessKey, token.tSecretKey, token.tToken, [object objectForKey:@"Expiration"]);
        
        return token;
      }else{
        return nil;
      }
      
    }
  }];
  if(self){
    self.authServerUrl = authServerUrl;
  }
  return self;
}

@end
